@@ -4,11 +4,11 @@
 
 ;; Author: Laurence Warne
 ;; Maintainer: Laurence Warne
-;; Package-Version: 0.5.0
-;; Package-Revision: v0.5.0-0-g5e20777922aa
+;; Package-Version: 20250907.1936
+;; Package-Revision: be1c866c288b
 ;; Keywords: outlines
 ;; URL: https://github.com/LaurenceWarne/finito.el
-;; Package-Requires: ((emacs "27.1") (dash "2.17.0") (request "0.3.2") (f "0.2.0") (s "1.12.0") (transient "0.3.0") (graphql "0.1.1") (async "1.9.3"))
+;; Package-Requires: ((emacs "27.1") (dash "2.19.1") (request "0.3.2") (f "0.2.0") (s "1.12.0") (transient "0.3.0") (graphql "0.1.1") (async "1.9.3"))
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -44,25 +44,21 @@
 (require 'dash)
 (require 'ewoc)
 (require 'f)
+(require 'json)
 (require 'org)
 (require 'outline)
 (require 'request)
 (require 's)
+(require 'subr-x)
+(require 'transient)
 
 (require 'finito-buffer)
 (require 'finito-core)
 (require 'finito-request)
 (require 'finito-server)
-(require 'finito-view)
 
 ;;; Custom variables
 
-(defcustom finito-writer-instance
-  (finito-book-writer)
-  "This object will be used to write books in finito buffers."
-  :group 'finito
-  :type 'object)
-
 (defcustom finito-img-cache-directory
   (f-join finito-server-directory "images/")
   "The directory of the finito image cache."
@@ -97,7 +93,7 @@ This instance will be used to initialise a buffer after a keyword search."
  This collection will be opened when the \"My Books\" suffix is invoked from
 the `finito' prefix command.  Typically this collection will hold all
 books which have been added at some point to some collection.  See
-URL 'https://github.com/LaurenceWarne/libro-finito#special-collections' for
+URL `https://github.com/LaurenceWarne/libro-finito#special-collections' for
 more information on how special collections can be configured."
   :group 'finito
   :type 'string)
@@ -155,6 +151,12 @@ montage-path"
   :group 'finito
   :type 'boolean)
 
+(defcustom finito-save-last-search
+  t
+  "If non-nil, save the arguments of the last `finito-search'."
+  :group 'finito
+  :type 'boolean)
+
 ;;; Constants
 
 (defconst finito--montage-large-image-width 128)
@@ -170,7 +172,9 @@ montage-path"
 
 (defvar finito--parallel-img-download t)
 
-;;; Misc functions
+(defvar finito--sort-asc-alist '(("ascending" . true) ("descending" . false)))
+
+;;; Internal functions
 
 (cl-defun finito--make-request
     (request-plist callback &key sync graphql-error)
@@ -180,7 +184,8 @@ CALLBACK is called with the parsed json if the request is successful.
 If SYNC is non-nil make the request synchronous.  If errors are detected
 in the graphql response body, then call GRAPHQL-ERROR with the first error
 as a symbol."
-  (request finito--host-uri
+  (when finito--debug (message "Finito request content: '%s'" request-plist))
+  (request (finito--host-uri)
     :type "POST"
     :headers (plist-get request-plist :headers)
     :data (plist-get request-plist :data)
@@ -197,8 +202,8 @@ as a symbol."
                       (let ((first-error (elt (cdadr data) 0)))
                         (if graphql-error
                             (--> (cdr (-last-item (-last-item first-error)))
-                              (s-dashed-words it)
-                              (funcall graphql-error (intern it)))
+                                 (s-dashed-words it)
+                                 (funcall graphql-error (intern it)))
                           ;; Error doesn't seem to do anything here
                           (message "Received error(s) in gql response: %s"
                                    (cdar first-error))))
@@ -208,7 +213,7 @@ as a symbol."
     :sync sync))
 
 (defun finito--process-books-data (data init-obj)
-  "Insert the books data DATA into a buffer.
+  "Insert the books data DATA into a buffer, and return the buffer.
 
 Use INIT-OBJ, an instance of `finito-buffer-info' to initialize the buffer."
   (make-directory finito-img-cache-directory t)
@@ -219,7 +224,9 @@ Use INIT-OBJ, an instance of `finito-buffer-info' to initialize the buffer."
                          ;; ewoc adds a newline after each node and after
                          ;; the footer even if it is empty
                          (goto-char (point-max))
-                         (delete-char -3))
+                         (while (s-blank-str-p (buffer-substring-no-properties
+                                                (1- (point)) (point)))
+                           (delete-char -1)))
                (proc-books () (finito--prepare-buffer init-obj #'callback)))
       (cond (finito-use-image-uris
              (proc-books))
@@ -230,32 +237,42 @@ Use INIT-OBJ, an instance of `finito-buffer-info' to initialize the buffer."
              (proc-books))))))
 
 (defun finito--prepare-buffer (init-obj callback)
-  "Prepare a finito buffer.
+  "Prepare a finito buffer and return it.
 
 Prepare a finito buffer using INIT-OBJ which should be a `finito-buffer-info'
 instance, then call CALLBACK with an ewoc, which should use it to insert text
 in some way, and then apply some final configuration to the buffer."
   (when (oref init-obj buf-name-unique)
     (ignore-errors (kill-buffer (oref init-obj buf-name))))
-  (switch-to-buffer (generate-new-buffer-name (oref init-obj buf-name)))
-  (finito-init-buffer init-obj)
-  (let ((inhibit-read-only t)
-        (buffer-ewoc
-         (ewoc-create
-          (lambda (obj) (finito-insert-book finito-writer-instance obj))
-          (finito-title-string init-obj))))
-    (setq finito--ewoc buffer-ewoc)
-    (funcall callback buffer-ewoc))
-  (let ((inhibit-message t))
-    (toggle-truncate-lines -1))
-  (goto-char (point-min))
-  (org-display-inline-images))
+  (let ((buf (generate-new-buffer (oref init-obj buf-name))))
+    (with-current-buffer buf
+      (finito--benchmark finito--debug "Buffer insertion took: %ss"
+        (finito-init-buffer init-obj)
+        (let ((inhibit-read-only t)
+              (buffer-ewoc
+               (ewoc-create
+                (lambda (obj) (finito-insert-book finito-writer-instance obj))
+                (finito-title-string init-obj))))
+          (setq finito--ewoc buffer-ewoc)
+          (funcall callback buffer-ewoc))
+        (let ((inhibit-message t))
+          (toggle-truncate-lines -1))
+        (goto-char (point-min))
+        (org-display-inline-images)))
+    (display-buffer buf '(display-buffer-same-window . nil))
+    buf))
+
+(defun finito--url-copy-file-log-error (url newname)
+  "Copy URL to NEWNAME like `url-copy-file', but catch and log any errors."
+  (condition-case nil
+      (url-copy-file url newname)
+    (error (message "Error downloading thumbnail '%s'" url))))
 
 (defun finito--download-images (books)
   "Download the images for BOOKS."
   (--each
       (--remove (f-exists-p (alist-get 'image-file-name it)) books)
-    (let-alist it (url-copy-file .img-uri .image-file-name))))
+    (let-alist it (finito--url-copy-file-log-error .img-uri .image-file-name))))
 
 (defun finito--download-images-par (books callback)
   "Download the images for BOOKS in parallel and then call CALLBACK."
@@ -269,7 +286,7 @@ in some way, and then apply some final configuration to the buffer."
                   'handles
                   (async-start
                    `(lambda ()
-                      (url-copy-file ,.img-uri ,.image-file-name)
+                      (ignore-errors (url-copy-file ,.img-uri ,.image-file-name))
                       ,.image-file-name)
                    (lambda (path)
                      (message "Downloaded '%s'" path)
@@ -290,6 +307,7 @@ isbn
 img-uri
 image-file-name
 rating
+review
 started-reading
 last-read"
   (let-alist book-response
@@ -304,6 +322,7 @@ last-read"
             (cons 'img-uri .thumbnailUri)
             (cons 'image-file-name image-file-name)
             (cons 'rating .rating)
+            (cons 'review .review)
             (cons 'started-reading .startedReading)
             (cons 'last-read .lastRead)))))
 
@@ -313,23 +332,7 @@ last-read"
 The returned alist will match the format returned by
 `finito--create-book-alist'."
   (unless (ewoc-nth finito--ewoc 0) (error "No books in the current buffer!"))
-  (ewoc-data (ewoc-locate finito--ewoc (point))))
-
-(defun finito--select-collection (callback &optional collection-filter sync)
-  "Prompt for a collection, and then call CALLBACK with that collection.
-
-If COLLECTION-FILTER is specified, only include collections in the prompt
-for which COLLECTION-FILTER applied to the collection name evaluates to a
-non-nil value.  If SYNC is non-nil, make the request synchronous."
-  (finito--make-request
-   (finito--collections-request-plist)
-   (lambda (response)
-     (let* ((all-collections (-filter (or collection-filter (-const t))
-                                      (-map #'cdar response)))
-            ;; TODO check if any collections exist first
-            (chosen-collection (completing-read "Choose: " all-collections)))
-       (funcall callback chosen-collection)))
-   :sync sync))
+  (ewoc-data (ewoc-locate finito--ewoc)))
 
 (defun finito--browse-function (book-alist)
   "Open an openlibrary page of a book, using the isbn in BOOK-ALIST.
@@ -338,29 +341,32 @@ BOOK-ALIST should be of the form returned by `finito--create-book-alist'."
   (browse-url
    (concat "https://openlibrary.org/isbn/" (alist-get 'isbn book-alist))))
 
-(defun finito--open-specified-collection (collection &optional sync offset)
-  "Open the collection COLLECTION in a view buffer.
+(defun finito--open-specified-collection (collection &optional sync offset callback)
+  "Open the collection COLLECTION in a view buffer and call CALLBACK.
 
 If SYNC it non-nil, perform all actions synchronously.
 OFFSET is the offset of the books in collection."
   (finito--make-request
-   (finito--collection-request-plist collection (or offset 0))
+   (finito--collection-request-plist
+    collection (and (finito--use-pagination) finito-collection-books-limit) (or offset 0))
    (lambda (response)
-     (finito--process-books-data
-      (cdar response)
-      (finito-collection-buffer-info
-       :title collection
-       :mode #'finito-collection-view-mode
-       :buf-name (concat "Collection: " collection)
-       :buf-name-unique t
-       :books-offset (or offset 0)
-       :total-books (or (ignore-errors (cdadar (last response))) 0))))
+     (with-current-buffer
+         (finito--process-books-data
+          (cdar response)
+          (finito-collection-buffer-info
+           :title collection
+           :mode #'finito-collection-view-mode
+           :buf-name (concat "Collection: " collection)
+           :buf-name-unique t
+           :books-offset (or offset 0)
+           :total-books (or (ignore-errors (cdadar (last response))) 0)))
+       (when callback (funcall callback))))
    :sync sync))
 
 (defun finito--remove-book-region ()
   "Remove the book at point from the current buffer."
   (let ((inhibit-read-only t))
-    (ewoc-delete finito--ewoc (ewoc-locate finito--ewoc (point)))))
+    (ewoc-delete finito--ewoc (ewoc-locate finito--ewoc))))
 
 (defun finito--replace-book-at-point-from-request
     (plist &optional success-message)
@@ -397,43 +403,88 @@ request is successful"
          ((_ _ from-year) (calendar-current-date -30)))
     (cons (format "%s-01-01" from-year) (format "%s-%02d-%02d" year month day))))
 
-(defun finito--set-show-description-for-collection (collection flag)
-  "Set show description to FLAG for COLLECTION."
-  (if (member collection (mapcar #'car finito-show-description-alist))
-      (setf (cdr (assoc finito--collection finito-show-description-alist)) flag)
-    (setq finito-show-description-alist
-          `((,collection . ,flag) . ,finito-show-description-alist))))
+(defun finito--set-show-attribute-for-collection (alist-sym alist collection flag)
+  "Set show the attribute implied by ALIST and ALIST-SYM to FLAG for COLLECTION."
+  (if (member collection (mapcar #'car alist))
+      (setf (cdr (assoc finito--collection alist)) flag)
+    (set alist-sym `((,collection . ,flag) . ,alist))))
 
 (defun finito--init-summary-buffer (summary-alist)
   "Create a summary buffer using data from SUMMARY-ALIST."
-  (switch-to-buffer (generate-new-buffer "finito summary"))
-  (finito-summary-mode)
-  (let ((inhibit-read-only t))
-    (let-alist summary-alist
-      (insert "* Year In Books\n\n")
-      (insert (format "[[%s]]\n\n" .montage-path))
-      (insert "** Your Statistics\n\n")
-      (insert (format "- You've read %d" .read))
-      (overlay-put
-       (make-overlay (- (point) (length (number-to-string .read))) (point))
-       'face 'finito-summary-read)
-      (insert (format " books in %d and added a total of %d." 2021 .added))
-      (overlay-put
-       (make-overlay (1- (- (point) (length (number-to-string .added))))
-                     (1- (point)))
-       'face 'finito-summary-added)
-      (let ((rating-str (if (= (floor .average-rating) .average-rating)
-                            (number-to-string (floor .average-rating))
-                          (format "%0.2f" .average-rating))))
-        (insert (format "\n- You gave an average rating of %s." rating-str))
-        (overlay-put
-         (make-overlay (- (point) (length (number-to-string .read)) 1)
-                       (- (point) 1))
-         'face 'finito-summary-average-rating))
-      (when finito-summary-show-recommended
-        (insert (concat "\n\n" finito--summary-recommended-text)))
-      (org-display-inline-images)
-      (goto-char (point-min)))))
+  (let ((buf (generate-new-buffer "finito summary")))
+    (with-current-buffer buf
+      (finito-summary-mode)
+      (let ((inhibit-read-only t))
+        (let-alist summary-alist
+          (insert "* Year In Books\n\n")
+          (insert (format "[[%s]]\n\n" .montage-path))
+          (insert "** Your Statistics\n\n")
+          (insert (format "- You've read %d" .read))
+          (overlay-put
+           (make-overlay (- (point) (length (number-to-string .read))) (point))
+           'face 'finito-summary-read)
+          (insert (format (format-time-string " books in %Y and added a total of %d.") .added))
+          (overlay-put
+           (make-overlay (1- (- (point) (length (number-to-string .added))))
+                         (1- (point)))
+           'face 'finito-summary-added)
+          (let ((rating-str (if (= (floor .average-rating) .average-rating)
+                                (number-to-string (floor .average-rating))
+                              (format "%0.2f" .average-rating))))
+            (insert (format "\n- You gave an average rating of %s." rating-str))
+            (overlay-put
+             (make-overlay (- (point) (length (number-to-string .read)) 1)
+                           (1- (point)))
+             'face 'finito-summary-average-rating))
+          (when finito-summary-show-recommended
+            (insert (concat "\n\n" finito--summary-recommended-text)))
+          (org-display-inline-images)
+          (goto-char (point-min)))))
+    (display-buffer buf '(display-buffer-same-window . nil))))
+
+(defun finito--ewoc-node-index (ewoc node)
+  "Return the index of NODE in EWOC."
+  (let* ((head-node (ewoc-nth ewoc 0))
+         (cur-node head-node)
+         (idx 0))
+    (while (and (not (eq cur-node node)) (or (zerop idx) (not (eq cur-node head-node))))
+      (setq cur-node (ewoc-next ewoc cur-node))
+      (cl-incf idx))
+    (and (eq node cur-node) idx)))
+
+(defun finito--extract-collections-from-response (response &optional collection-filter)
+  "Extract collections data from RESPONSE.
+
+Output collections are further filtered by COLLECTION-FILTER."
+  (-filter (or collection-filter (-const t))
+           (-map #'cdar response)))
+
+(defun finito--select-collection (callback &optional collection-filter sync)
+  "Prompt for a collection, and then call CALLBACK with that collection.
+
+If COLLECTION-FILTER is specified, only include collections in the prompt
+for which COLLECTION-FILTER applied to the collection name evaluates to a
+non-nil value.  If SYNC is non-nil, make the request synchronous."
+  (finito--make-request
+   (finito--collections-request-plist)
+   (lambda (response)
+     (let* ((all-collections (finito--extract-collections-from-response
+                              response
+                              collection-filter))
+            ;; TODO check if any collections exist first
+            (chosen-collection (completing-read "Choose: " all-collections)))
+       (funcall callback chosen-collection)))
+   :sync sync))
+
+(defun finito--transient-select-collection (_prompt _initial-input _history)
+  "Choose a collection."
+  (let ((response (finito--make-request
+                   (finito--collections-request-plist)
+                   #'ignore
+                   :sync t)))
+    (completing-read "Choose: "
+                     (finito--extract-collections-from-response
+                      (alist-get 'collections (alist-get 'data (request-response-data response)))))))
 
 ;;; Modes
 
@@ -457,6 +508,11 @@ request is successful"
     (define-key map "e" #'finito-series-at-point)
     (define-key map "w" #'finito-title-of-book-at-point)
     (define-key map "d" #'finito-toggle-show-descriptions)
+    (define-key map "W" #'finito-toggle-show-reviews)
+    (define-key map "M" #'finito-toggle-minimal)
+    (define-key map "g" #'revert-buffer)
+    (define-key map "l" #'finito-replay-search)
+    (define-key map "R" #'finito-review-book-at-point)
     (define-key map (kbd "C-m") #'finito-open-my-books-collection)
     (define-key map (kbd "C-r") #'finito-open-currently-reading-collection)
     map))
@@ -473,7 +529,6 @@ The following commands are available in this mode:
 (defvar finito-search-view-mode-map
   (let ((map (make-sparse-keymap)))
     (suppress-keymap map t)
-    (define-key map "l" #'finito-replay-search)
     map))
 
 (define-derived-mode finito-search-view-mode
@@ -484,12 +539,12 @@ The following commands are available in this mode:
 The following commands are available in this mode:
 \\{finito-search-view-mode-map}"
   (use-local-map finito-search-view-mode-map)
+  (setq-local revert-buffer-function #'finito-search-revert)
   (setq-local finito--show-descriptions finito-show-descriptions-default))
 
 (defvar finito-collection-view-mode-map
   (let ((map (make-sparse-keymap)))
     (suppress-keymap map t)
-    (define-key map "g" #'revert-buffer)
     (define-key map "D" #'finito-remove-book-at-point)
     (define-key map "N" #'finito-collection-next)
     (define-key map "P" #'finito-collection-previous)
@@ -555,7 +610,7 @@ The following commands are available in this mode:
 
 Copy to the kill ring a curl request string corresponding to what the server
 would send to the Google Books API (see URL
-'https://developers.google.com/books/docs/v1/using') for the same
+`https://developers.google.com/books/docs/v1/using') for the same
 set of args.  This is intended to be used for debugging."
   (interactive
    (list (finito--transient-args-plist 'finito-search)))
@@ -569,7 +624,7 @@ set of args.  This is intended to be used for debugging."
 
 Copy to the kill ring a curl request string corresponding to what the server
 would send to the Google Books API (see URL
-'https://developers.google.com/books/docs/v1/using') for a keyword
+`https://developers.google.com/books/docs/v1/using') for a keyword
 search using TITLE and AUTHOR keywords.  This is intended to be used for
 debugging."
   (unless (or title author)
@@ -598,7 +653,7 @@ debugging."
 
 Copy to the kill ring a curl request string corresponding to what the server
 would send to the Google Books API (see URL
-'https://developers.google.com/books/docs/v1/using') for the given ISBN.
+`https://developers.google.com/books/docs/v1/using') for the given ISBN.
 This is intended to be used for debugging."
   (let* ((base-url "https://www.googleapis.com/books/v1/volumes?q=")
          (url-params (list (concat "isbn:" isbn)
@@ -694,6 +749,8 @@ maximum of MAX-RESULTS results."
                               nil
                               nil
                               #'string=)))
+     (when (xor sort-asc preferred-sort)
+       (user-error "Both of sort ascending and preferred sort must be set if at least one is"))
      (finito--make-request
       (finito--update-collection-request-plist
        chosen-collection new-name preferred-sort sort-asc)
@@ -713,7 +770,7 @@ maximum of MAX-RESULTS results."
   (interactive)
   (let ((buf (generate-new-buffer "finito org")))
     (copy-to-buffer buf (point-min) (point-max))
-    (switch-to-buffer buf)
+    (display-buffer buf '(display-buffer-same-window . nil))
     (org-display-inline-images)))
 
 (defun finito-add-book-at-point ()
@@ -747,16 +804,18 @@ maximum of MAX-RESULTS results."
   (interactive)
   (finito--wait-for-server-then
    (let* ((book (finito--book-at-point))
+          (title (alist-get 'title book))
           (isbn (alist-get 'isbn book))
           (line (line-number-at-pos))
           (buf (current-buffer)))
-     (finito--make-request
-      (finito--remove-book-request-plist finito--collection isbn)
-      (lambda (_)
-        (message "Removed '%s' from %s"
-                 (alist-get 'title book)
-                 finito--collection)
-        (finito--goto-buffer-line-and-remove-book-at-point buf line))))))
+     (when (y-or-n-p (format "Remove '%s' from %s?" title finito--collection))
+       (finito--make-request
+        (finito--remove-book-request-plist finito--collection isbn)
+        (lambda (_)
+          (message "Removed '%s' from %s"
+                   (alist-get 'title book)
+                   finito--collection)
+          (finito--goto-buffer-line-and-remove-book-at-point buf line)))))))
 
 (defun finito-collection-revert (&optional _ignore-auto _noconfirm)
   "Refresh the current collection, _IGNORE-AUTO and _NOCONFIRM are ignored."
@@ -764,11 +823,28 @@ maximum of MAX-RESULTS results."
   (finito--wait-for-server-then
    (let ((collection finito--collection)
          (old-point (point))
+         (node-idx (or (ignore-errors
+                         (finito--ewoc-node-index finito--ewoc (ewoc-locate finito--ewoc)))
+                       0))
          (request-backend 'url-retrieve))
      (kill-current-buffer)
-     (finito--open-specified-collection collection t)
-     (goto-char old-point)
-     (message "Refreshed collection '%s'" collection))))
+     (finito--open-specified-collection
+      collection
+      t
+      nil
+      (lambda ()
+        (unless (ignore-errors (ewoc-goto-node
+                                finito--ewoc (ewoc-nth finito--ewoc node-idx)))
+          (goto-char (min old-point (point-max))))
+        (message "Refreshed collection '%s'" collection))))))
+
+(defun finito-search-revert (&optional _ignore-auto _noconfirm)
+  "Refresh the current search, _IGNORE-AUTO and _NOCONFIRM are ignored."
+  (interactive)
+  (let ((node (ewoc-locate finito--ewoc)))
+    (ewoc-refresh finito--ewoc)
+    (ewoc-goto-node finito--ewoc node))
+  (org-display-inline-images))
 
 (define-obsolete-function-alias
   'finito-refresh-collection
@@ -792,6 +868,33 @@ maximum of MAX-RESULTS results."
               (alist-get 'title book)
               rating)))))
 
+(defun finito-review-book-at-point ()
+  "Add a review for the book at point.
+
+Note this overwrites any existing review."
+  (interactive)
+  (finito--wait-for-server-then
+   (let* ((book (finito--book-at-point))
+          (coll-buf (current-buffer))
+          (buf (generate-new-buffer (format "%s Review" (alist-get 'title book)))))
+     (with-current-buffer buf
+       ;; TODO make a major mode
+       (local-set-key
+        (kbd "C-c C-c")
+        (lambda () (interactive)
+          (finito--make-request
+           (finito--review-book-request-plist
+            book
+            (json-encode-string (buffer-substring-no-properties (point-min) (point-max))))
+           (lambda (&rest _)
+             (with-current-buffer coll-buf (revert-buffer))
+             (format "Added review for '%s'" (alist-get 'title book))))
+          (kill-buffer-and-window)))
+       (when-let ((review (alist-get 'review book)))
+         (insert review)))
+     (pop-to-buffer buf '(display-buffer-below-selected . nil))
+     (message "Use C-c C-c to submit the review"))))
+
 (defun finito-start-book-at-point (&optional date)
   "Mark the book at point as currently reading.
 
@@ -838,11 +941,15 @@ When DATE is specified, mark that as the date the book was finished."
   (interactive)
   (finito--wait-for-server-then
    (let ((book (finito--book-at-point)))
-     (finito--make-request
-      (finito--delete-book-data-request-plist (alist-get 'isbn book))
-      (lambda (_)
-        (message "Deleted info held about '%s'" (alist-get 'title book))
-        (finito-collection-revert))))))
+     (when (y-or-n-p
+            (format
+             "Remove held data for '%s' (note this won't remove the book from any collections)?"
+             (alist-get 'title book)))
+       (finito--make-request
+        (finito--delete-book-data-request-plist (alist-get 'isbn book))
+        (lambda (_)
+          (message "Deleted info held about '%s'" (alist-get 'title book))
+          (finito-collection-revert)))))))
 
 (defun finito-replay-search ()
   "Open the search transient prefix with the last args that were used."
@@ -853,20 +960,22 @@ When DATE is specified, mark that as the date the book was finished."
 (defun finito-collection-next ()
   "Get the next page of books for the current collection."
   (interactive)
-  (let ((new-offset (+ finito--collection-books-current-offset
-                       finito-collection-books-limit)))
-    (if (>= new-offset finito--collection-total-books)
-        (message "Already at the end of %s!" finito--collection)
-      (finito--open-specified-collection finito--collection nil new-offset))))
+  (when (finito--use-pagination)
+    (let ((new-offset (+ finito--collection-books-current-offset
+                         finito-collection-books-limit)))
+      (if (>= new-offset finito--collection-total-books)
+          (message "Already at the end of %s!" finito--collection)
+        (finito--open-specified-collection finito--collection nil new-offset)))))
 
 (defun finito-collection-previous ()
   "Get the previous page of books for the current collection."
   (interactive)
-  (let ((new-offset (- finito--collection-books-current-offset
-                       finito-collection-books-limit)))
-    (if (< new-offset 0)
-        (message "Already at the start of %s!" finito--collection)
-      (finito--open-specified-collection finito--collection nil new-offset))))
+  (when (finito--use-pagination)
+    (let ((new-offset (- finito--collection-books-current-offset
+                         finito-collection-books-limit)))
+      (if (< new-offset 0)
+          (message "Already at the start of %s!" finito--collection)
+        (finito--open-specified-collection finito--collection nil new-offset)))))
 
 ;;;###autoload
 (defun finito-create-book (title authors description img-uri isbn)
@@ -915,11 +1024,18 @@ sPlease input a unique identifier (used in place of an isbn):")
            finito-keyword-search-buffer-init-instance)))))))
 
 ;;;###autoload
-(defun finito-summary ()
-  "Open a summary buffer of reading highlights in the past year."
+(defun finito-summary (&optional from to)
+  "Open a summary buffer of reading highlights in the past year.
+
+If either FROM or TO are specified, use these dates instead of the default of
+the past year.
+
+Example:
+`(let ((finito-montage-image-columns 16))
+   (finito-summary \"2021-01-01\" \"2099-01-01\"))'."
   (interactive)
-  (-let (((from . to) (finito--get-summary-from-to)))
-    (finito-summary-in-period from to)))
+  (-let (((default-from . default-to) (finito--get-summary-from-to)))
+    (finito-summary-in-period (or from default-from) (or to default-to))))
 
 ;;;###autoload
 (defun finito-summary-in-period (&optional from to)
@@ -962,6 +1078,7 @@ sPlease input a unique identifier (used in place of an isbn):")
      (kill-new title)
      (message "Copied '%s' to the kill ring" title))))
 
+;; The setq-local make these next two hard to DRY
 (defun finito-toggle-show-descriptions ()
   "Toggle display of descriptions."
   (interactive)
@@ -973,8 +1090,11 @@ sPlease input a unique identifier (used in place of an isbn):")
                                 nil
                                 'equal))))
     (cond ((bound-and-true-p finito--collection)
-           (finito--set-show-description-for-collection finito--collection
-                                                        (not alist-val)))
+           (finito--set-show-attribute-for-collection
+            'finito-show-description-alist
+            finito-show-description-alist
+            finito--collection
+            (not alist-val)))
           ((boundp 'finito--show-descriptions)
            (setq finito--show-descriptions (not local-val)))
           (t (setq-local finito--show-descriptions
@@ -984,5 +1104,215 @@ sPlease input a unique identifier (used in place of an isbn):")
       (ewoc-goto-node finito--ewoc node))
     (org-display-inline-images)))
 
+(defun finito-toggle-show-reviews ()
+  "Toggle display of reviews."
+  (interactive)
+  (let ((local-val (bound-and-true-p finito--show-reviews))
+        (alist-val (when (bound-and-true-p finito--collection)
+                     (alist-get finito--collection
+                                finito-show-review-alist
+                                finito-show-reviews-default
+                                nil
+                                'equal))))
+    (cond ((bound-and-true-p finito--collection)
+           (finito--set-show-attribute-for-collection
+            'finito-show-review-alist
+            finito-show-review-alist
+            finito--collection
+            (not alist-val)))
+          ((boundp 'finito--show-reviews)
+           (setq finito--show-reviews (not local-val)))
+          (t (setq-local finito--show-reviews
+                         (not finito-show-reviews-default))))
+    (let ((node (ewoc-locate finito--ewoc)))
+      (ewoc-refresh finito--ewoc)
+      (ewoc-goto-node finito--ewoc node))
+    (org-display-inline-images)))
+
+(defun finito-toggle-minimal ()
+  "Toggle the book writer instance to/from a minimal book writer."
+  (interactive)
+  (setq finito-writer-instance
+        (if (finito-minimal-book-writer-p finito-writer-instance)
+            finito-detailed-writer-instance
+          finito-minimal-writer-instance))
+  (revert-buffer))
+
+(defun finito-goodreads-import (csv-file)
+  "Import the Goodreads export CSV-FILE.
+
+In order to create an export, visit URL `https://www.goodreads.com/review/import'."
+  (interactive "fSelect output CSV file: ")
+  (finito--wait-for-server-then
+   (message "Making request, this may take a while for large files!")
+   (finito--make-request
+    (finito--goodreads-import-request-plist
+     (f-read-text csv-file))
+    (lambda (response)
+      (let-alist response
+        (message "Finito import complete, Successful: %s, partially successful: %s, failed: %s"
+                 (length .sucessful)
+                 (length .partiallySuccessful)
+                 (length .unsuccessful)))))))
+
+;;; Transients
+
+(defclass finito--transient-argument (transient-argument)
+  ((plist-key :initarg :plist-key)))
+
+(cl-defmethod transient-format-value ((obj finito--transient-argument))
+  "Format OBJ and return the result."
+  (let ((value (oref obj value)))
+    (propertize (if (listp value) (mapconcat #'identity value ",")
+                  value)
+                'face (if value
+                          'transient-value
+                        'transient-inactive-value))))
+
+(cl-defmethod transient-init-value ((obj finito--transient-argument))
+  "Initialize the value of OBJ using the value of the current prefix."
+  (let* ((prefix-plist (oref transient--prefix value))
+         (val-or-nil (plist-get prefix-plist (oref obj plist-key))))
+    (oset obj value val-or-nil)))
+
+(defclass finito--search-prefix (transient-prefix) nil)
+
+(cl-defmethod transient--history-push ((obj finito--search-prefix))
+  "Push the current value of OBJ to its entry in `transient-history'."
+  (let ((key (transient--history-key obj)))
+    (setf (alist-get key transient-history)
+          (let ((args (finito--transient-args-plist (oref obj command))))
+            (cons args (delete args (alist-get key transient-history)))))))
+
+(cl-defmethod transient-init-value :after ((obj finito--search-prefix))
+  "Set the value of OBJ from history if applicable.
+
+If OBJ has an empty value and `finito-save-last-search' is non-nil, switch
+to the last value used for OBJ."
+  (transient--history-init obj)
+  (when (and finito-save-last-search
+             (not (oref obj value))
+             (cdr (oref obj history)))
+    (let ((transient--prefix obj))
+      (transient-history-prev))))
+
+;;; Infix Arguments
+
+(transient-define-argument finito--title-arg ()
+  :class 'finito--transient-argument
+  :key "t"
+  :plist-key ':title
+  :argument "title=")
+
+(transient-define-argument finito--author-arg ()
+  :class 'finito--transient-argument
+  :key "a"
+  :plist-key ':author
+  :argument "author=")
+
+(transient-define-argument finito--isbn-arg ()
+  :class 'finito--transient-argument
+  :key "i"
+  :plist-key ':isbn
+  :argument "isbn=")
+
+(transient-define-argument finito--max-results-arg ()
+  :class 'finito--transient-argument
+  :key "n"
+  :plist-key ':max-results
+  :argument "max results="
+  :reader #'transient-read-number-N+)
+
+(transient-define-argument finito--collection-name-arg ()
+  :class 'finito--transient-argument
+  :key "c"
+  :plist-key ':name
+  :argument "name="
+  :reader #'finito--transient-select-collection)
+
+(transient-define-argument finito--new-collection-name-arg ()
+  :class 'finito--transient-argument
+  :key "n"
+  :plist-key ':new-name
+  :argument "new name=")
+
+(transient-define-argument finito--sort-arg ()
+  :class 'finito--transient-argument
+  :key "s"
+  :plist-key ':sort
+  :argument "Sort="
+  :choices '("Date Added" "Author" "Title"))
+
+(transient-define-argument finito--sort-asc ()
+  :class 'finito--transient-argument
+  :key "o"
+  :plist-key ':sort-ascending
+  :argument "Sort Ascending="
+  :choices (-map #'car finito--sort-asc-alist))
+
+;;; Prefixes
+
+;;;###autoload (autoload 'finito "finito-view" nil t)
+(transient-define-prefix finito ()
+  "Search for books."
+  ["Actions"
+   ("m" "📚 My Books"           finito-open-my-books-collection)
+   ("r" "📖 Currently Reading"  finito-open-currently-reading-collection)
+   ("R" "📕 Read"               finito-open-read-collection)
+   ("s" "🔍 Search"             finito-search)
+   ("o" "📁 Open a Collection"  finito-open-collection)
+   ("c" "🚐 Collection Actions" finito-collection)])
+
+(transient-define-prefix finito-search ()
+  "Search for books."
+  :class 'finito--search-prefix
+  :incompatible '(("isbn=" "author=") ("isbn=" "title=") ("isbn=" "max results="))
+  ["By Keywords"
+   (finito--title-arg :description "Title" :prompt "Title: " :always-read t)
+   (finito--author-arg :description "Author" :prompt "Author: " :always-read t)
+   (finito--max-results-arg :description "Max Results" :prompt "Max results: " :always-read t)]
+  ["Direct Lookup"
+   (finito--isbn-arg :description "ISBN" :prompt "ISBN: " :always-read t)]
+  ["Actions"
+   ("C" "Copy Curl"     finito-search-request-curl-dbg)
+   ("s" "Search"        finito-search-request)])
+
+(transient-define-prefix finito-collection ()
+  "Search for books."
+  ["Actions"
+   ("n" "Create a new Collection" finito-create-collection)
+   ("o" "Open a Collection"       finito-open-collection)
+   ("u" "Update a Collection"     finito-update-collection)
+   ("d" "Delete a Collection"     finito-delete-collection)
+   ("i" "Import a Collection"     ignore)
+   ("e" "Export a Collection"     ignore)])
+
+(transient-define-prefix finito-update-collection ()
+  "Search for books."
+  ["Attributes"
+   ;; TODO read this using `finito--select-collection'
+   (finito--collection-name-arg
+    :description "Name"
+    :prompt "Name: ")
+   (finito--new-collection-name-arg
+    :description "New Name"
+    :prompt "New name: ")
+   (finito--sort-arg
+    :description "Sort Books By"
+    :prompt "Sort Books By: ")
+   (finito--sort-asc
+    :description "Sort Ascending"
+    :prompt "Ascending: ")]
+  ["Actions"
+   ("u" "Update" finito-update-collection-request)])
+
+;; Has to be defined here to pass byte-compile for some reason
+(defun finito--transient-args-plist (prefix)
+  "Return the infixes of PREFIX as a plist."
+  (-flatten-n 1 (--map (list (oref it plist-key)
+                             (oref it value))
+                       (-filter #'finito--transient-argument-p
+                                (transient-suffixes prefix)))))
+
 (provide 'finito)
 ;;; finito.el ends here
